/*
 * Copyright (C) 2017  Kevin Whitaker <eyecreate@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "PlayerInterface.h"
#include <Wt/WText>
#include "../GroovePlayer.h"
#include <grooveplayer/player.h>
#include <Wt/WLength>
#include <Wt/WColor>
#include <Wt/WBorder>

PlayerInterface::PlayerInterface(WebInterface* app)
{
    this->app = app;
    interfaceLayout = new Wt::WVBoxLayout();
    this->setLayout(interfaceLayout);
    playControlLayout = new Wt::WHBoxLayout();
    voteControlLayout = new Wt::WVBoxLayout();
    playControlWidget = new Wt::WContainerWidget();
    playControlWidget->setLayout(playControlLayout);
    
    currentTrackProgress = new Wt::WProgressBar();
    currentTrackProgress->decorationStyle().setBackgroundColor(Wt::WColor("#B20A5E"));
    currentTrackProgress->setFormat("");
    trackProgress = new Wt::WTimer();
    trackProgress->setInterval(1000);
    trackProgress->timeout().connect(this,&PlayerInterface::updateProgressFromTimer);
    currentTrackDetails = new TrackDetails();
    playpause = new Wt::WPushButton();
    playpause->setText(groove_playlist_playing(GroovePlayerMgr::getInstance()->currentPlaylist)==1?"Pause":"Resume");
    playpause->setMaximumSize(Wt::WLength(60,Wt::WLength::Point),Wt::WLength(30,Wt::WLength::Point));
    isPaused = groove_playlist_playing(GroovePlayerMgr::getInstance()->currentPlaylist)==0;
    playpause->clicked().connect(this, &PlayerInterface::playpauseClicked);
    
    interfaceLayout->addWidget(playControlWidget);
    interfaceLayout->addWidget(currentTrackProgress);
    interfaceLayout->addLayout(voteControlLayout);
    playControlWidget->decorationStyle().setBorder(Wt::WBorder::Outset);
    playControlWidget->decorationStyle().setBackgroundColor(Wt::WColor("#00B200"));
    playControlLayout->addWidget(playpause,0,Wt::AlignmentFlag::AlignLeft | Wt::AlignmentFlag::AlignMiddle);
    playControlLayout->addWidget(currentTrackDetails);
}

void PlayerInterface::playpauseClicked()
{
    if(!isPaused) 
    {
        GroovePlayerMgr::getInstance()->lastInternalEvents.push_back(GroovePlayerMgr::PlayerEvent() = {GroovePlayerMgr::PlayerEventType::PLAYING_PAUSED, app->currentUser});
    }
    else
    {
        GroovePlayerMgr::getInstance()->lastInternalEvents.push_back(GroovePlayerMgr::PlayerEvent() = {GroovePlayerMgr::PlayerEventType::PLAYING_RESUMED, app->currentUser});
    }
}

void PlayerInterface::playpauseUpdated()
{
    if(isPaused) 
    {
        playpause->setText("Resume");
    }
    else
    {
        playpause->setText("Pause");
    }
}

void PlayerInterface::updateProgressFromTimer()
{
    if(currentTrackProgress->maximum() >= currentTrackProgress->value()+1)
    {
        double secondsIn;
        groove_player_position(GroovePlayerMgr::getInstance()->currentPlayer,nullptr, &secondsIn);
        currentTrackProgress->setValue(secondsIn);
    }
    else if(currentTrackProgress->maximum() < currentTrackProgress->value()+1)
    {
        currentTrackProgress->setValue(currentTrackProgress->maximum());
        trackProgress->stop();
    }
}

void PlayerInterface::updateDetailsFromServer(AudioTrack track)
{
    //Find current song playing and set progress state.
    double secondsIn;
    groove_player_position(GroovePlayerMgr::getInstance()->currentPlayer,nullptr, &secondsIn);
    currentTrackProgress->setMaximum(track.trackLengthSeconds);
    currentTrackProgress->setValue(secondsIn);
    isPaused = groove_playlist_playing(GroovePlayerMgr::getInstance()->currentPlaylist) == 0;
    if(!isPaused)
    {
        trackProgress->start();
    }
    playpauseUpdated();
    currentTrackDetails->updateWithTrackDetails(track);
}
